unit DataRead;

interface

uses
  SysUtils;


procedure CheckForPortBaseAddr;
procedure Port_InitStatus;
procedure Port_InitCommands;
procedure Port_SetBaseAdress(Idx: byte);
procedure Data_ReadSetToError(bState: boolean);
function  Data_CaptureAllInputs: boolean;
function  Data_ReadAllBlock(bAr: array of boolean): boolean;
function  Data_ReadBlock(iBlock: byte): boolean;
function  LPT_CaptureAllInputs: boolean;
function  LPT_ReadAllBlock(bAr: array of boolean): boolean;
function  LPT_ReadBlock(iBlock: byte): boolean;
function  COM_CaptureAllInputs: boolean;
function  COM_ReadAllBlock(bAr: array of boolean): boolean;
function  COM_ReadBlock(iBlock: byte): boolean;
function  JOY_CaptureAllInputs: boolean;
function  JOY_ReadAllBlock(bAr: array of boolean): boolean;
function  JOY_ReadBlock(iBlock: byte): boolean;

var
  ActivePortBaseAddr: integer;    // actually used port base address

  // if used with LPT, then I0=Busy, I1=Ack, I2=PaperEnd, I3=SelectI, I4=Error
  // if used with COM, then I0=CTS, I1=DSR, I2=RI, I3=DCD, all others not used
  // if used with JOY, then I0=, I1=, I2=, I3=, I4=
  I0, I1, I2, I3, I4: boolean;

  // if used with LPT, then O0=D0, O1=D1, O2=D2, O3=D3, O4=D4, O5=D5, O6=D6, O7=D7
  // if used with COM, then O0=DTR, O1=RTS, O2=TX, all others not used
  // if used with JOY, then O0=, O1=, O2=, O3=, O4=, O5=, O6=, O7=
  O0, O1, O2, O3, O4, O5, O6, O7: boolean;

  // if used with LPT, then O8=Strobe, O9=Autofeed, O10=Init, O11=SelectO
  // never used with COM
  // never used with JOY
  O8, O9, O10, O11: boolean;

implementation

uses
  rmMisc, rmPorts, Main;


procedure CheckForPortBaseAddr;
begin
  if (ActivePortBaseAddr <> LPT1PortBaseAddr) and
     (ActivePortBaseAddr <> LPT2PortBaseAddr) and
     (ActivePortBaseAddr <> COM1PortBaseAddr) and
     (ActivePortBaseAddr <> COM2PortBaseAddr) and
     (ActivePortBaseAddr <> COM3PortBaseAddr) and
     (ActivePortBaseAddr <> COM4PortBaseAddr) and
     (ActivePortBaseAddr <> JoystickPortBaseAddr) then begin
    Data_ReadSetToError(true);
    abort;
  end;
end;

procedure Port_InitStatus;
begin
  I0 {Busy} := false;
  I1 {Ack} := false;
  I2 {PaperEnd} := false;
  I3 {SelectI} := false;
  I4 {Error} := false;
end;

procedure Port_InitCommands;
begin
  O0 {D0} := false;
  O1 {D1} := false;
  O2 {D2} := false;
  O3 {D3} := false;
  O4 {D4} := false;
  O5 {D5} := false;
  O6 {D6} := false;
  O7 {D7} := false;
  O8 {Strobe} := false;
  O9 {Autofeed} := false;
  O10 {Init} := false;
  O11 {SelectO} := false;
end;

procedure Port_SetBaseAdress(Idx: byte);
begin
  case Idx of
    1 : ActivePortBaseAddr := LPT1PortBaseAddr;
    2 : ActivePortBaseAddr := LPT2PortBaseAddr;
    3 : ActivePortBaseAddr := COM1PortBaseAddr;
    4 : ActivePortBaseAddr := COM2PortBaseAddr;
    5 : ActivePortBaseAddr := COM3PortBaseAddr;
    6 : ActivePortBaseAddr := COM4PortBaseAddr;
    7 : ActivePortBaseAddr := JoystickPortBaseAddr;
  end;
end;

procedure Data_ReadSetToError(bState: boolean);
begin
  if bState then begin
    fmMain.lblError.Visible := true;
    fmMain.ActCaptureStopExecute(nil);
  end;
end;

function Data_CaptureAllInputs: boolean;
begin

  // verify selected port base address
  CheckForPortBaseAddr;

  // read from selected port
  case ActivePortToUse of
    apLPT1..apLPT2 : LPT_CaptureAllInputs;
    apCOM1..apCOM4 : COM_CaptureAllInputs;
    apJOY1         : JOY_CaptureAllInputs;
  end;

end;

function Data_ReadAllBlock(bAr: array of boolean): boolean;
begin

  // verify selected port base address
  CheckForPortBaseAddr;

  // read from selected port
  case ActivePortToUse of
    apLPT1..apLPT2 : LPT_ReadAllBlock(bAr);
    apCOM1..apCOM4 : COM_ReadAllBlock(bAr);
    apJOY1         : JOY_ReadAllBlock(bAr);
  end;

end;

function Data_ReadBlock(iBlock: byte): boolean;
begin

  // verify selected port base address
  CheckForPortBaseAddr;

  // read from selected port
  case ActivePortToUse of
    apLPT1..apLPT2 : LPT_ReadBlock(iBlock);
    apCOM1..apCOM4 : COM_ReadBlock(iBlock);
    apJOY1         : JOY_ReadBlock(iBlock);
  end;

end;

function LPT_CaptureAllInputs: boolean;
var
  bState, ReadOK: boolean;
begin

  // O8=Strobe, O9=Autofeed, O10=Init, O11=SelectO

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  bState := true;
  O9 := bState;
  try
    ReadOK := ParallelPort_WriteControlValues(ActivePortBaseAddr, O8, O9, O10, O11);
    O9 := not bState;
    if ReadOK then
      ReadOK := ParallelPort_WriteControlValues(ActivePortBaseAddr, O8, O9, O10, O11);
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function COM_CaptureAllInputs: boolean;
var
  bState, ReadOK: boolean;
begin

  // do nothing at the moment
  exit;

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  try
    ReadOK := false;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function JOY_CaptureAllInputs: boolean;
var
  bState, ReadOK: boolean;
begin

  // do nothing at the moment
  exit;

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  try
    ReadOK := false;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function LPT_ReadAllBlock(bAr: array of boolean): boolean;
var
  ReadOK: boolean;
  i, dGroup, dBlock: integer;
  bData: array[0..3] of boolean;
begin


  // O0=D0, O1=D1, O2=D2, O3=D3, O4=D4, O5=D5, O6=D6, O7=D7

  // verify selected port base address
  CheckForPortBaseAddr;

  // group 0..7 : Analog, group 8..15 : Digital, 8 inputs / group
  // input 0..63 : Analog, input 64..127 : Digital
  // 1 block = 4 inputs
  for dBlock := 0 to 15 do
  begin
    dGroup := (dBlock div 2) + 8;
    if not Groups[dBlock].Active then
      continue;
    // select block to read via hardware demux
    IntegerToArrayBool(dBlock, bData);
    O0 := bData[0];
    O1 := bData[1];
    O2 := bData[2];
    O3 := bData[3];
    ReadOK := ParallelPort_WriteDataValues(ActivePortBaseAddr, O0, O1, O2, O3, O4, O5, O6, O7);
    // read block
    if ReadOK then
      ReadOK := Data_ReadBlock(dBlock);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function COM_ReadAllBlock(bAr: array of boolean): boolean;
var
  ReadOK: boolean;
begin

  // do nothing at the moment
  exit;

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  try
    ReadOK := false;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function JOY_ReadAllBlock(bAr: array of boolean): boolean;
var
  ReadOK: boolean;
begin

  // do nothing at the moment
  exit;

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  try
    ReadOK := false;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function LPT_ReadBlock(iBlock: byte): boolean;
var
  ReadOK: boolean;
  BlockIdx: integer;
begin
  // I0=Busy, I1=Ack, I2=PaperEnd, I3=SelectI, I4=Error

  // verify selected port base address
  CheckForPortBaseAddr;

  try
    ReadOK := ParallelPort_ReadStatusValues(ActivePortBaseAddr, I0, I1, I2, I3, I4);
    BlockIdx := iBlock * 4;
    Inputs[BlockIdx + 0].ActualState := I1;
    Inputs[BlockIdx + 1].ActualState := I2;
    Inputs[BlockIdx + 2].ActualState := I3;
    Inputs[BlockIdx + 3].ActualState := I4;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function COM_ReadBlock(iBlock: byte): boolean;
var
  ReadOK: boolean;
begin

  // do nothing at the moment
  exit;

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  try
    ReadOK := false;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;

function JOY_ReadBlock(iBlock: byte): boolean;
var
  ReadOK: boolean;
begin

  // do nothing at the moment
  exit;

  // verify selected port base address
  CheckForPortBaseAddr;

  // load all input in flip-flop via Autofeed line
  try
    ReadOK := false;
  except
    Data_ReadSetToError(true);
  end;
  result := ReadOK;
  if not ReadOK then
    Data_ReadSetToError(true);
end;


end.
