unit InputConfig;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ToolWin, IniFiles, Buttons,
  rmCommutator, Menus, lmdstdcA;

type
  TfmInputConfig = class(TForm)
    Panel1: TPanel;
    Label6: TLabel;
    rmCommutator1: TrmCommutator;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    pnlInput: TPanel;
    cbActiveAlarm: TCheckBox;
    Label2: TLabel;
    edDelayBeforeAlarm: TLMDSpinEdit;
    edCountBeforeAlarm: TLMDSpinEdit;
    Label5: TLabel;
    lblErrorDesc: TLabel;
    lblGroupCaption: TLabel;
    edGroupCaption: TEdit;
    edInputCaption: TEdit;
    lblInputCaption: TLabel;
    btnResetAlarm: TSpeedButton;
    MainMenu1: TMainMenu;
    Fichier1: TMenuItem;
    mnuClose: TMenuItem;
    Edition1: TMenuItem;
    mnuCopy: TMenuItem;
    mnuPaste: TMenuItem;
    cbEnabled: TCheckBox;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    pnlValues: TPanel;
    Label4: TLabel;
    Label24: TLabel;
    Label3: TLabel;
    edMaxLimit: TEdit;
    edMinLimit: TEdit;
    edActualValue: TEdit;
    pnlStatus: TPanel;
    Label25: TLabel;
    Label1: TLabel;
    edActualStatus: TEdit;
    rbNominalStateOFF: TRadioButton;
    rbNominalStateON: TRadioButton;
    Label14: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure edGroupCaptionChange(Sender: TObject);
    procedure rmCommutator1Change(Sender: TObject);
    procedure btnResetAlarmClick(Sender: TObject);
    procedure mnuCloseClick(Sender: TObject);
    procedure mnuCopyClick(Sender: TObject);
    procedure mnuPasteClick(Sender: TObject);
  private
    { Dclarations prives }
    GroupIndex, InputIndex: integer;
    SelectedInputIndex: integer;
    ReadDataInProgress: boolean;
    procedure GetParameters(SelInputIdx: integer);
    procedure SetParameters(SelInputIdx: integer);
  public
    { Dclarations publiques }
  end;

var
  fmInputConfig: TfmInputConfig;

implementation

uses
  ResStr, rmMisc, Main;

{$R *.DFM}


procedure TfmInputConfig.FormCreate(Sender: TObject);
begin
  ReadDataInProgress := false;
  SelectedInputIndex := 0;
end;

procedure TfmInputConfig.rmCommutator1Change(Sender: TObject);
begin
  GroupIndex := rmCommutator1.selectedLine; // 0..7
  InputIndex := rmCommutator1.selectedRow;  // 0 to 15
  SelectedInputIndex := (16 * GroupIndex) + InputIndex;
  GetParameters(SelectedInputIndex);
end;

procedure TfmInputConfig.GetParameters(SelInputIdx: integer);
begin
  ReadDataInProgress := true;

  // refresh name of group
  case GroupIndex of
    0..3 : lblGroupCaption.Caption := format(sGroupNameAnalog, [intToStr(GroupIndex + 1)]);
    4..7 : lblGroupCaption.Caption := format(sGroupNameDigital, [intToStr(GroupIndex + 1 - 4)]);
  end;
  edGroupCaption.Text := Groups[GroupIndex].Caption;

  // activate and deactivate controls used for analog or digital inputs
  // and refresh input values or states
  lblInputCaption.Caption := format(sInputName, [intToStr(InputIndex + 1)]);
  edInputCaption.Text := Inputs[SelInputIdx].Caption;
  cbEnabled.Checked := Inputs[SelInputIdx].Enabled;
  rbNominalStateOn.enabled := GroupIndex in [4..7];
  rbNominalStateOff.enabled := GroupIndex in [4..7];
  edMinLimit.enabled := GroupIndex in [0..3]; // only used on analog input
  edMinLimit.Color := boolToColor(GroupIndex in [0..3], clWindow, clBtnFace);
  edMaxLimit.enabled := GroupIndex in [0..3]; // only used on analog input
  edMaxLimit.Color := boolToColor(GroupIndex in [0..3], clWindow, clBtnFace);
  cbActiveAlarm.Checked := Inputs[SelInputIdx].ActiveAlarm;

  pnlValues.Visible := GroupIndex in [0..3];
  pnlStatus.Visible := GroupIndex in [4..7];

  case GroupIndex of
    0..3 : begin  // analog inputs
      edActualStatus.Text := '';
      rbNominalStateOn.Checked := false;
      rbNominalStateOff.Checked := false;
      edActualValue.text := realToStr(3, Inputs[SelInputIdx].ActualValue);
      edMinLimit.Text := realToStr(3, Inputs[SelInputIdx].MinLimit);
      edMaxLimit.Text :=  realToStr(3, Inputs[SelInputIdx].MaxLimit);
      cbActiveAlarm.Caption := 'Alarme si dpassement des limites';
    end;
    4..7 : begin // digital inputs
      edActualStatus.text := BoolToStr(Inputs[SelInputIdx].ActualState, sON, sOFF);
      if Inputs[SelInputIdx].NominalStateIsOn then
        rbNominalStateOn.Checked := true
      else
        rbNominalStateOff.Checked := true;
      edActualValue.text := '';
      edMinLimit.Text := '';
      edMaxLimit.Text := '';
      cbActiveAlarm.Caption := 'Alarme si changement de statut';
    end;
  end;
  edDelayBeforeAlarm.value := Inputs[SelInputIdx].DelayBeforeAlarm;
  edCountBeforeAlarm.value := Inputs[SelInputIdx].CountBeforeAlarm;

  // refresh input state
  lblErrorDesc.caption := format('[%s] %s', [intToStr(Inputs[SelInputIdx].errorCode), fmMain.ErrorCodeToStr(Inputs[SelInputIdx].errorCode)]);

  ReadDataInProgress := false;

end;

procedure TfmInputConfig.edGroupCaptionChange(Sender: TObject);
begin
  SetParameters(SelectedInputIndex);
  if sender = cbEnabled then
    pnlInput.visible := cbEnabled.checked;
end;

procedure TfmInputConfig.SetParameters(SelInputIdx: integer);
var
  sVal: string;
begin

  if ReadDataInProgress then exit;

  // only for group
  Groups[groupIndex].caption := edGroupCaption.text;

  // common to analog and digital inputs
  Inputs[SelInputIdx].Enabled := cbEnabled.Checked;
  Inputs[SelInputIdx].Caption := edInputCaption.text;
  Inputs[SelInputIdx].DelayBeforeAlarm := edDelayBeforeAlarm.value;
  Inputs[SelInputIdx].CountBeforeAlarm := edCountBeforeAlarm.value;
  Inputs[SelInputIdx].ActiveAlarm := cbActiveAlarm.Checked;
  // Inputs[SelInputIdx].ErrorCode := 0;


  case SelInputIdx of
    000..063 : begin  // only for analog input
      sVal := edMinLimit.text;
      GetGoodDecimalSeparator(sVal);
      Inputs[SelInputIdx].MinLimit := strToFloat(sVal);
      sVal := edMaxLimit.text;
      GetGoodDecimalSeparator(sVal);
      Inputs[SelInputIdx].MaxLimit := strToFloat(sVal);
    end;
    064..127 : begin  // only for digital input
      Inputs[SelInputIdx].NominalStateIsOn := rbNominalStateON.checked;
    end;
  end;

  ActualConfigModified := true;
end;

procedure TfmInputConfig.btnResetAlarmClick(Sender: TObject);
begin
  Inputs[selectedInputIndex].ErrorCode := 0;
  GetParameters(SelectedInputIndex);
end;

procedure TfmInputConfig.mnuCloseClick(Sender: TObject);
begin
  close;
end;

procedure TfmInputConfig.mnuCopyClick(Sender: TObject);
begin
  ClipBrdInput := Inputs[selectedInputIndex];
end;

procedure TfmInputConfig.mnuPasteClick(Sender: TObject);
var
  sCaption: string;
begin
  // copy all TInput record fields, except input caption
  sCaption := Inputs[selectedInputIndex].Caption;
  Inputs[selectedInputIndex] := ClipBrdInput;
  Inputs[selectedInputIndex].Caption := sCaption;
  GetParameters(SelectedInputIndex);
end;

end.
